# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

"""Auto-generated package that contains models used by the AccelByte Gaming Services Platform Service."""

__version__ = "4.30.2"
__author__ = "AccelByte"
__email__ = "dev@accelbyte.net"

# pylint: disable=line-too-long

from .achievement import Achievement
from .achievement_info import AchievementInfo
from .additional_data import AdditionalData
from .admin_order_create import AdminOrderCreate
from .admin_order_create import PlatformEnum as AdminOrderCreatePlatformEnum
from .adyen_config import AdyenConfig
from .ali_pay_config import AliPayConfig
from .app_entitlement_info import AppEntitlementInfo
from .app_entitlement_info import (
    AppTypeEnum as AppEntitlementInfoAppTypeEnum,
    StatusEnum as AppEntitlementInfoStatusEnum,
)
from .app_entitlement_paging_sliced_result import AppEntitlementPagingSlicedResult
from .app_info import AppInfo
from .app_info import (
    GenresEnum as AppInfoGenresEnum,
    PlatformsEnum as AppInfoPlatformsEnum,
    PlayersEnum as AppInfoPlayersEnum,
    PrimaryGenreEnum as AppInfoPrimaryGenreEnum,
)
from .app_localization import AppLocalization
from .app_update import AppUpdate
from .app_update import (
    GenresEnum as AppUpdateGenresEnum,
    PlatformsEnum as AppUpdatePlatformsEnum,
    PlayersEnum as AppUpdatePlayersEnum,
    PrimaryGenreEnum as AppUpdatePrimaryGenreEnum,
)
from .apple_iap_config_info import AppleIAPConfigInfo
from .apple_iap_config_request import AppleIAPConfigRequest
from .apple_iap_receipt import AppleIAPReceipt
from .available_comparison import AvailableComparison
from .available_comparison import ComparisonEnum as AvailableComparisonComparisonEnum
from .available_predicate import AvailablePredicate
from .available_predicate import (
    PredicateTypeEnum as AvailablePredicatePredicateTypeEnum,
    ValueTypeEnum as AvailablePredicateValueTypeEnum,
)
from .basic_category_info import BasicCategoryInfo
from .basic_item import BasicItem
from .basic_item import (
    AppTypeEnum as BasicItemAppTypeEnum,
    EntitlementTypeEnum as BasicItemEntitlementTypeEnum,
    ItemTypeEnum as BasicItemItemTypeEnum,
    SeasonTypeEnum as BasicItemSeasonTypeEnum,
    StatusEnum as BasicItemStatusEnum,
)
from .billing_account import BillingAccount
from .billing_account import PaymentProviderEnum as BillingAccountPaymentProviderEnum
from .billing_history_info import BillingHistoryInfo
from .billing_history_info import StatusEnum as BillingHistoryInfoStatusEnum
from .billing_history_paging_sliced_result import BillingHistoryPagingSlicedResult
from .box_item import BoxItem
from .bulk_credit_request import BulkCreditRequest
from .bulk_credit_result import BulkCreditResult
from .bulk_credit_result import StatusEnum as BulkCreditResultStatusEnum
from .bulk_debit_request import BulkDebitRequest
from .bulk_debit_result import BulkDebitResult
from .bulk_debit_result import StatusEnum as BulkDebitResultStatusEnum
from .bulk_entitlement_grant_request import BulkEntitlementGrantRequest
from .bulk_entitlement_grant_result import BulkEntitlementGrantResult
from .bulk_entitlement_grant_result import (
    StatusEnum as BulkEntitlementGrantResultStatusEnum,
)
from .bulk_entitlement_revoke_result import BulkEntitlementRevokeResult
from .bulk_entitlement_revoke_result import (
    StatusEnum as BulkEntitlementRevokeResultStatusEnum,
)
from .bulk_operation_result import BulkOperationResult
from .bulk_region_data_change_request import BulkRegionDataChangeRequest
from .bundled_item_info import BundledItemInfo
from .bundled_item_info import (
    AppTypeEnum as BundledItemInfoAppTypeEnum,
    EntitlementTypeEnum as BundledItemInfoEntitlementTypeEnum,
    ItemTypeEnum as BundledItemInfoItemTypeEnum,
    SeasonTypeEnum as BundledItemInfoSeasonTypeEnum,
    StatusEnum as BundledItemInfoStatusEnum,
)
from .campaign_create import CampaignCreate
from .campaign_create import (
    RedeemTypeEnum as CampaignCreateRedeemTypeEnum,
    StatusEnum as CampaignCreateStatusEnum,
    TypeEnum as CampaignCreateTypeEnum,
)
from .campaign_dynamic_info import CampaignDynamicInfo
from .campaign_info import CampaignInfo
from .campaign_info import (
    RedeemTypeEnum as CampaignInfoRedeemTypeEnum,
    StatusEnum as CampaignInfoStatusEnum,
    TypeEnum as CampaignInfoTypeEnum,
)
from .campaign_paging_sliced_result import CampaignPagingSlicedResult
from .campaign_update import CampaignUpdate
from .campaign_update import (
    RedeemTypeEnum as CampaignUpdateRedeemTypeEnum,
    StatusEnum as CampaignUpdateStatusEnum,
)
from .cancel_request import CancelRequest
from .catalog_change_info import CatalogChangeInfo
from .catalog_change_info import (
    ActionEnum as CatalogChangeInfoActionEnum,
    ItemTypeEnum as CatalogChangeInfoItemTypeEnum,
    StatusEnum as CatalogChangeInfoStatusEnum,
    TypeEnum as CatalogChangeInfoTypeEnum,
)
from .catalog_change_paging_sliced_result import CatalogChangePagingSlicedResult
from .catalog_change_statistics import CatalogChangeStatistics
from .category_create import CategoryCreate
from .category_info import CategoryInfo
from .category_update import CategoryUpdate
from .checkout_config import CheckoutConfig
from .client_request_parameter import ClientRequestParameter
from .code_create import CodeCreate
from .code_create_result import CodeCreateResult
from .code_info import CodeInfo
from .code_info import (
    RedeemTypeEnum as CodeInfoRedeemTypeEnum,
    StatusEnum as CodeInfoStatusEnum,
    TypeEnum as CodeInfoTypeEnum,
)
from .code_info_paging_sliced_result import CodeInfoPagingSlicedResult
from .condition_group import ConditionGroup
from .condition_group import OperatorEnum as ConditionGroupOperatorEnum
from .condition_group_validate_result import ConditionGroupValidateResult
from .condition_match_result import ConditionMatchResult
from .consumable_entitlement_revocation_config import (
    ConsumableEntitlementRevocationConfig,
)
from .consumable_entitlement_revocation_config import (
    StrategyEnum as ConsumableEntitlementRevocationConfigStrategyEnum,
)
from .consume_item import ConsumeItem
from .consume_item import ItemIdentityTypeEnum as ConsumeItemItemIdentityTypeEnum
from .credit_request import CreditRequest
from .credit_request import (
    OriginEnum as CreditRequestOriginEnum,
    SourceEnum as CreditRequestSourceEnum,
)
from .credit_result import CreditResult
from .credit_revocation import CreditRevocation
from .credit_revocation import StatusEnum as CreditRevocationStatusEnum
from .credit_summary import CreditSummary
from .currency_config import CurrencyConfig
from .currency_create import CurrencyCreate
from .currency_create import CurrencyTypeEnum as CurrencyCreateCurrencyTypeEnum
from .currency_info import CurrencyInfo
from .currency_info import CurrencyTypeEnum as CurrencyInfoCurrencyTypeEnum
from .currency_summary import CurrencySummary
from .currency_summary import CurrencyTypeEnum as CurrencySummaryCurrencyTypeEnum
from .currency_update import CurrencyUpdate
from .currency_wallet import CurrencyWallet
from .customization import Customization
from .debit_by_currency_code_request import DebitByCurrencyCodeRequest
from .debit_by_currency_code_request import (
    BalanceOriginEnum as DebitByCurrencyCodeRequestBalanceOriginEnum,
)
from .debit_request import DebitRequest
from .debit_result import DebitResult
from .delete_reward_condition_request import DeleteRewardConditionRequest
from .detailed_wallet_transaction_info import DetailedWalletTransactionInfo
from .detailed_wallet_transaction_info import (
    WalletActionEnum as DetailedWalletTransactionInfoWalletActionEnum,
)
from .detailed_wallet_transaction_paging_sliced_result import (
    DetailedWalletTransactionPagingSlicedResult,
)
from .dlc_item import DLCItem
from .dlc_item_config_info import DLCItemConfigInfo
from .dlc_item_config_update import DLCItemConfigUpdate
from .dlc_record import DLCRecord
from .dlc_record import StatusEnum as DLCRecordStatusEnum
from .durable_entitlement_revocation_config import DurableEntitlementRevocationConfig
from .durable_entitlement_revocation_config import (
    StrategyEnum as DurableEntitlementRevocationConfigStrategyEnum,
)
from .entitlement_decrement import EntitlementDecrement
from .entitlement_decrement_result import EntitlementDecrementResult
from .entitlement_decrement_result import (
    AppTypeEnum as EntitlementDecrementResultAppTypeEnum,
    ClazzEnum as EntitlementDecrementResultClazzEnum,
    SourceEnum as EntitlementDecrementResultSourceEnum,
    StatusEnum as EntitlementDecrementResultStatusEnum,
    TypeEnum as EntitlementDecrementResultTypeEnum,
)
from .entitlement_grant import EntitlementGrant
from .entitlement_grant import SourceEnum as EntitlementGrantSourceEnum
from .entitlement_grant_result import EntitlementGrantResult
from .entitlement_history_info import EntitlementHistoryInfo
from .entitlement_history_info import ActionEnum as EntitlementHistoryInfoActionEnum
from .entitlement_info import EntitlementInfo
from .entitlement_info import (
    AppTypeEnum as EntitlementInfoAppTypeEnum,
    ClazzEnum as EntitlementInfoClazzEnum,
    SourceEnum as EntitlementInfoSourceEnum,
    StatusEnum as EntitlementInfoStatusEnum,
    TypeEnum as EntitlementInfoTypeEnum,
)
from .entitlement_loot_box_reward import EntitlementLootBoxReward
from .entitlement_ownership import EntitlementOwnership
from .entitlement_paging_sliced_result import EntitlementPagingSlicedResult
from .entitlement_revocation import EntitlementRevocation
from .entitlement_revocation import StatusEnum as EntitlementRevocationStatusEnum
from .entitlement_revocation_config import EntitlementRevocationConfig
from .entitlement_revoke_result import EntitlementRevokeResult
from .entitlement_sold_request import EntitlementSoldRequest
from .entitlement_sold_result import EntitlementSoldResult
from .entitlement_summary import EntitlementSummary
from .entitlement_summary import (
    ClazzEnum as EntitlementSummaryClazzEnum,
    TypeEnum as EntitlementSummaryTypeEnum,
)
from .entitlement_update import EntitlementUpdate
from .entitlement_update import StatusEnum as EntitlementUpdateStatusEnum
from .epic_games_dlc_sync_request import EpicGamesDLCSyncRequest
from .epic_games_iap_config_info import EpicGamesIAPConfigInfo
from .epic_games_iap_config_request import EpicGamesIAPConfigRequest
from .epic_games_reconcile_request import EpicGamesReconcileRequest
from .epic_games_reconcile_result import EpicGamesReconcileResult
from .epic_games_reconcile_result import (
    StatusEnum as EpicGamesReconcileResultStatusEnum,
)
from .error_entity import ErrorEntity
from .event_payload import EventPayload
from .export_store_request import ExportStoreRequest
from .extension_fulfillment_summary import ExtensionFulfillmentSummary
from .extension_fulfillment_summary import (
    ItemTypeEnum as ExtensionFulfillmentSummaryItemTypeEnum,
)
from .external_payment_order_create import ExternalPaymentOrderCreate
from .external_payment_order_create import (
    ItemTypeEnum as ExternalPaymentOrderCreateItemTypeEnum,
)
from .field_validation_error import FieldValidationError
from .fixed_period_rotation_config import FixedPeriodRotationConfig
from .fixed_period_rotation_config import (
    BackfillTypeEnum as FixedPeriodRotationConfigBackfillTypeEnum,
    RuleEnum as FixedPeriodRotationConfigRuleEnum,
)
from .fulfill_code_request import FulfillCodeRequest
from .fulfillment_error import FulfillmentError
from .fulfillment_history_info import FulfillmentHistoryInfo
from .fulfillment_history_info import StatusEnum as FulfillmentHistoryInfoStatusEnum
from .fulfillment_history_paging_sliced_result import (
    FulfillmentHistoryPagingSlicedResult,
)
from .fulfillment_item import FulfillmentItem
from .fulfillment_item import ItemTypeEnum as FulfillmentItemItemTypeEnum
from .fulfillment_request import FulfillmentRequest
from .fulfillment_request import (
    OriginEnum as FulfillmentRequestOriginEnum,
    SourceEnum as FulfillmentRequestSourceEnum,
)
from .fulfillment_result import FulfillmentResult
from .fulfillment_script_context import FulfillmentScriptContext
from .fulfillment_script_context import SourceEnum as FulfillmentScriptContextSourceEnum
from .fulfillment_script_create import FulfillmentScriptCreate
from .fulfillment_script_eval_test_request import FulfillmentScriptEvalTestRequest
from .fulfillment_script_eval_test_request import (
    TypeEnum as FulfillmentScriptEvalTestRequestTypeEnum,
)
from .fulfillment_script_eval_test_result import FulfillmentScriptEvalTestResult
from .fulfillment_script_info import FulfillmentScriptInfo
from .fulfillment_script_update import FulfillmentScriptUpdate
from .full_app_info import FullAppInfo
from .full_app_info import (
    GenresEnum as FullAppInfoGenresEnum,
    PlatformsEnum as FullAppInfoPlatformsEnum,
    PlayersEnum as FullAppInfoPlayersEnum,
    PrimaryGenreEnum as FullAppInfoPrimaryGenreEnum,
)
from .full_category_info import FullCategoryInfo
from .full_item_info import FullItemInfo
from .full_item_info import (
    AppTypeEnum as FullItemInfoAppTypeEnum,
    EntitlementTypeEnum as FullItemInfoEntitlementTypeEnum,
    ItemTypeEnum as FullItemInfoItemTypeEnum,
    SeasonTypeEnum as FullItemInfoSeasonTypeEnum,
    StatusEnum as FullItemInfoStatusEnum,
)
from .full_item_paging_sliced_result import FullItemPagingSlicedResult
from .full_section_info import FullSectionInfo
from .full_section_info import RotationTypeEnum as FullSectionInfoRotationTypeEnum
from .full_view_info import FullViewInfo
from .google_iap_config_info import GoogleIAPConfigInfo
from .google_iap_config_request import GoogleIAPConfigRequest
from .google_iap_receipt import GoogleIAPReceipt
from .google_receipt_resolve_result import GoogleReceiptResolveResult
from .grant_subscription_days_request import GrantSubscriptionDaysRequest
from .hierarchical_category_info import HierarchicalCategoryInfo
from .iap_consume_history_info import IAPConsumeHistoryInfo
from .iap_consume_history_info import (
    IapTypeEnum as IAPConsumeHistoryInfoIapTypeEnum,
    StatusEnum as IAPConsumeHistoryInfoStatusEnum,
)
from .iap_consume_history_paging_sliced_result import (
    IAPConsumeHistoryPagingSlicedResult,
)
from .iap_item_config_info import IAPItemConfigInfo
from .iap_item_config_update import IAPItemConfigUpdate
from .iap_item_entry import IAPItemEntry
from .iap_item_entry import ItemIdentityTypeEnum as IAPItemEntryItemIdentityTypeEnum
from .iap_item_flat_entry import IAPItemFlatEntry
from .iap_item_flat_entry import (
    ItemIdentityTypeEnum as IAPItemFlatEntryItemIdentityTypeEnum,
    PlatformEnum as IAPItemFlatEntryPlatformEnum,
)
from .iap_item_mapping_info import IAPItemMappingInfo
from .iap_order_info import IAPOrderInfo
from .iap_order_info import (
    StatusEnum as IAPOrderInfoStatusEnum,
    TypeEnum as IAPOrderInfoTypeEnum,
)
from .iap_order_paging_sliced_result import IAPOrderPagingSlicedResult
from .image import Image
from .import_error_details import ImportErrorDetails
from .import_store_error import ImportStoreError
from .import_store_error import TypeEnum as ImportStoreErrorTypeEnum
from .import_store_item_info import ImportStoreItemInfo
from .import_store_item_info import ItemTypeEnum as ImportStoreItemInfoItemTypeEnum
from .import_store_result import ImportStoreResult
from .in_game_item_sync import InGameItemSync
from .invoice_currency_summary import InvoiceCurrencySummary
from .invoice_summary import InvoiceSummary
from .item_acquire_request import ItemAcquireRequest
from .item_acquire_result import ItemAcquireResult
from .item_create import ItemCreate
from .item_create import (
    AppTypeEnum as ItemCreateAppTypeEnum,
    EntitlementTypeEnum as ItemCreateEntitlementTypeEnum,
    ItemTypeEnum as ItemCreateItemTypeEnum,
    SeasonTypeEnum as ItemCreateSeasonTypeEnum,
    StatusEnum as ItemCreateStatusEnum,
)
from .item_dynamic_data_info import ItemDynamicDataInfo
from .item_id import ItemId
from .item_id import StatusEnum as ItemIdStatusEnum
from .item_info import ItemInfo
from .item_info import (
    AppTypeEnum as ItemInfoAppTypeEnum,
    EntitlementTypeEnum as ItemInfoEntitlementTypeEnum,
    ItemTypeEnum as ItemInfoItemTypeEnum,
    SeasonTypeEnum as ItemInfoSeasonTypeEnum,
    StatusEnum as ItemInfoStatusEnum,
)
from .item_naming import ItemNaming
from .item_naming import (
    ItemTypeEnum as ItemNamingItemTypeEnum,
    SeasonTypeEnum as ItemNamingSeasonTypeEnum,
    StatusEnum as ItemNamingStatusEnum,
)
from .item_paging_sliced_result import ItemPagingSlicedResult
from .item_purchase_condition_validate_request import (
    ItemPurchaseConditionValidateRequest,
)
from .item_purchase_condition_validate_result import ItemPurchaseConditionValidateResult
from .item_return_request import ItemReturnRequest
from .item_revocation import ItemRevocation
from .item_revocation import (
    ItemTypeEnum as ItemRevocationItemTypeEnum,
    StatusEnum as ItemRevocationStatusEnum,
)
from .item_snapshot import ItemSnapshot
from .item_snapshot import (
    AppTypeEnum as ItemSnapshotAppTypeEnum,
    EntitlementTypeEnum as ItemSnapshotEntitlementTypeEnum,
    ItemTypeEnum as ItemSnapshotItemTypeEnum,
    SeasonTypeEnum as ItemSnapshotSeasonTypeEnum,
)
from .item_type_config_create import ItemTypeConfigCreate
from .item_type_config_create import ItemTypeEnum as ItemTypeConfigCreateItemTypeEnum
from .item_type_config_info import ItemTypeConfigInfo
from .item_type_config_info import ItemTypeEnum as ItemTypeConfigInfoItemTypeEnum
from .item_type_config_update import ItemTypeConfigUpdate
from .item_update import ItemUpdate
from .item_update import (
    AppTypeEnum as ItemUpdateAppTypeEnum,
    EntitlementTypeEnum as ItemUpdateEntitlementTypeEnum,
    ItemTypeEnum as ItemUpdateItemTypeEnum,
    SeasonTypeEnum as ItemUpdateSeasonTypeEnum,
    StatusEnum as ItemUpdateStatusEnum,
)
from .key_group_create import KeyGroupCreate
from .key_group_create import StatusEnum as KeyGroupCreateStatusEnum
from .key_group_dynamic_info import KeyGroupDynamicInfo
from .key_group_info import KeyGroupInfo
from .key_group_info import StatusEnum as KeyGroupInfoStatusEnum
from .key_group_paging_sliced_result import KeyGroupPagingSlicedResult
from .key_group_update import KeyGroupUpdate
from .key_group_update import StatusEnum as KeyGroupUpdateStatusEnum
from .key_info import KeyInfo
from .key_info import StatusEnum as KeyInfoStatusEnum
from .key_paging_slice_result import KeyPagingSliceResult
from .list_view_info import ListViewInfo
from .localization import Localization
from .loot_box_config import LootBoxConfig
from .loot_box_config import RollFunctionEnum as LootBoxConfigRollFunctionEnum
from .loot_box_reward import LootBoxReward
from .loot_box_reward import TypeEnum as LootBoxRewardTypeEnum
from .mock_iap_receipt import MockIAPReceipt
from .mock_iap_receipt import (
    ItemIdentityTypeEnum as MockIAPReceiptItemIdentityTypeEnum,
    TypeEnum as MockIAPReceiptTypeEnum,
)
from .notification_process_result import NotificationProcessResult
from .notification_process_result import (
    StatusEnum as NotificationProcessResultStatusEnum,
)
from .option_box_config import OptionBoxConfig
from .order import Order
from .order import (
    PaymentProviderEnum as OrderPaymentProviderEnum,
    StatusEnum as OrderStatusEnum,
)
from .order_create import OrderCreate
from .order_creation_options import OrderCreationOptions
from .order_grant_info import OrderGrantInfo
from .order_history_info import OrderHistoryInfo
from .order_history_info import ActionEnum as OrderHistoryInfoActionEnum
from .order_info import OrderInfo
from .order_info import (
    PaymentProviderEnum as OrderInfoPaymentProviderEnum,
    StatusEnum as OrderInfoStatusEnum,
)
from .order_paging_result import OrderPagingResult
from .order_paging_sliced_result import OrderPagingSlicedResult
from .order_refund_create import OrderRefundCreate
from .order_statistics import OrderStatistics
from .order_summary import OrderSummary
from .order_sync_result import OrderSyncResult
from .order_update import OrderUpdate
from .order_update import StatusEnum as OrderUpdateStatusEnum
from .ownership import Ownership
from .ownership_token import OwnershipToken
from .paging import Paging
from .pay_pal_config import PayPalConfig
from .payment_account import PaymentAccount
from .payment_account import TypeEnum as PaymentAccountTypeEnum
from .payment_callback_config_info import PaymentCallbackConfigInfo
from .payment_callback_config_update import PaymentCallbackConfigUpdate
from .payment_merchant_config_info import PaymentMerchantConfigInfo
from .payment_method import PaymentMethod
from .payment_method import PaymentProviderEnum as PaymentMethodPaymentProviderEnum
from .payment_notification_info import PaymentNotificationInfo
from .payment_notification_info import (
    NotificationSourceEnum as PaymentNotificationInfoNotificationSourceEnum,
    StatusEnum as PaymentNotificationInfoStatusEnum,
)
from .payment_notification_paging_sliced_result import (
    PaymentNotificationPagingSlicedResult,
)
from .payment_order import PaymentOrder
from .payment_order import (
    ChannelEnum as PaymentOrderChannelEnum,
    ItemTypeEnum as PaymentOrderItemTypeEnum,
    PaymentProviderEnum as PaymentOrderPaymentProviderEnum,
    StatusEnum as PaymentOrderStatusEnum,
)
from .payment_order_charge_request import PaymentOrderChargeRequest
from .payment_order_charge_request import (
    PaymentProviderEnum as PaymentOrderChargeRequestPaymentProviderEnum,
)
from .payment_order_charge_status import PaymentOrderChargeStatus
from .payment_order_charge_status import (
    StatusEnum as PaymentOrderChargeStatusStatusEnum,
)
from .payment_order_create import PaymentOrderCreate
from .payment_order_create import ItemTypeEnum as PaymentOrderCreateItemTypeEnum
from .payment_order_create_result import PaymentOrderCreateResult
from .payment_order_create_result import (
    StatusEnum as PaymentOrderCreateResultStatusEnum,
)
from .payment_order_details import PaymentOrderDetails
from .payment_order_info import PaymentOrderInfo
from .payment_order_info import (
    ChannelEnum as PaymentOrderInfoChannelEnum,
    ItemTypeEnum as PaymentOrderInfoItemTypeEnum,
    PaymentProviderEnum as PaymentOrderInfoPaymentProviderEnum,
    StatusEnum as PaymentOrderInfoStatusEnum,
)
from .payment_order_notify_simulation import PaymentOrderNotifySimulation
from .payment_order_notify_simulation import (
    NotifyTypeEnum as PaymentOrderNotifySimulationNotifyTypeEnum,
    PaymentProviderEnum as PaymentOrderNotifySimulationPaymentProviderEnum,
)
from .payment_order_paging_sliced_result import PaymentOrderPagingSlicedResult
from .payment_order_paid_result import PaymentOrderPaidResult
from .payment_order_refund import PaymentOrderRefund
from .payment_order_refund_result import PaymentOrderRefundResult
from .payment_order_refund_result import (
    StatusEnum as PaymentOrderRefundResultStatusEnum,
)
from .payment_order_sync_result import PaymentOrderSyncResult
from .payment_process_result import PaymentProcessResult
from .payment_provider_config_edit import PaymentProviderConfigEdit
from .payment_provider_config_edit import (
    AggregateEnum as PaymentProviderConfigEditAggregateEnum,
    SpecialsEnum as PaymentProviderConfigEditSpecialsEnum,
)
from .payment_provider_config_info import PaymentProviderConfigInfo
from .payment_provider_config_info import (
    AggregateEnum as PaymentProviderConfigInfoAggregateEnum,
    SpecialsEnum as PaymentProviderConfigInfoSpecialsEnum,
)
from .payment_provider_config_paging_sliced_result import (
    PaymentProviderConfigPagingSlicedResult,
)
from .payment_request import PaymentRequest
from .payment_request import WalletPlatformEnum as PaymentRequestWalletPlatformEnum
from .payment_tax_config_edit import PaymentTaxConfigEdit
from .payment_tax_config_info import PaymentTaxConfigInfo
from .payment_token import PaymentToken
from .payment_url import PaymentUrl
from .payment_url import (
    PaymentProviderEnum as PaymentUrlPaymentProviderEnum,
    PaymentTypeEnum as PaymentUrlPaymentTypeEnum,
)
from .payment_url_create import PaymentUrlCreate
from .payment_url_create import (
    PaymentProviderEnum as PaymentUrlCreatePaymentProviderEnum,
)
from .platform_dlc_config_info import PlatformDLCConfigInfo
from .platform_dlc_config_update import PlatformDLCConfigUpdate
from .platform_dlc_entry import PlatformDlcEntry
from .platform_dlc_entry import PlatformEnum as PlatformDlcEntryPlatformEnum
from .platform_reward import PlatformReward
from .platform_reward import TypeEnum as PlatformRewardTypeEnum
from .platform_reward_currency import PlatformRewardCurrency
from .platform_reward_item import PlatformRewardItem
from .platform_subscribe_request import PlatformSubscribeRequest
from .platform_wallet import PlatformWallet
from .platform_wallet import (
    StatusEnum as PlatformWalletStatusEnum,
    WalletStatusEnum as PlatformWalletWalletStatusEnum,
)
from .platform_wallet_config_info import PlatformWalletConfigInfo
from .platform_wallet_config_update import PlatformWalletConfigUpdate
from .platform_wallet_config_update import (
    AllowedBalanceOriginsEnum as PlatformWalletConfigUpdateAllowedBalanceOriginsEnum,
)
from .play_station_dlc_sync_multi_service_labels_request import (
    PlayStationDLCSyncMultiServiceLabelsRequest,
)
from .play_station_dlc_sync_request import PlayStationDLCSyncRequest
from .play_station_iap_config_info import PlayStationIAPConfigInfo
from .play_station_multi_service_labels_reconcile_request import (
    PlayStationMultiServiceLabelsReconcileRequest,
)
from .play_station_reconcile_request import PlayStationReconcileRequest
from .play_station_reconcile_result import PlayStationReconcileResult
from .play_station_reconcile_result import (
    StatusEnum as PlayStationReconcileResultStatusEnum,
)
from .playstation_iap_config_request import PlaystationIAPConfigRequest
from .populated_item_info import PopulatedItemInfo
from .populated_item_info import (
    AppTypeEnum as PopulatedItemInfoAppTypeEnum,
    EntitlementTypeEnum as PopulatedItemInfoEntitlementTypeEnum,
    ItemTypeEnum as PopulatedItemInfoItemTypeEnum,
    SeasonTypeEnum as PopulatedItemInfoSeasonTypeEnum,
    StatusEnum as PopulatedItemInfoStatusEnum,
)
from .predicate import Predicate
from .predicate import (
    ComparisonEnum as PredicateComparisonEnum,
    PredicateTypeEnum as PredicatePredicateTypeEnum,
)
from .predicate_validate_result import PredicateValidateResult
from .purchase_condition import PurchaseCondition
from .purchase_condition_update import PurchaseConditionUpdate
from .purchased_item_count import PurchasedItemCount
from .recurring import Recurring
from .recurring import CycleEnum as RecurringCycleEnum
from .recurring_charge_result import RecurringChargeResult
from .redeem_history_info import RedeemHistoryInfo
from .redeem_history_paging_sliced_result import RedeemHistoryPagingSlicedResult
from .redeem_request import RedeemRequest
from .redeem_result import RedeemResult
from .redeemable_item import RedeemableItem
from .region_data_change import RegionDataChange
from .region_data_change import (
    ItemIdentityTypeEnum as RegionDataChangeItemIdentityTypeEnum,
)
from .region_data_item import RegionDataItem
from .region_data_item import CurrencyTypeEnum as RegionDataItemCurrencyTypeEnum
from .region_data_item_dto import RegionDataItemDTO
from .region_data_item_dto import CurrencyTypeEnum as RegionDataItemDTOCurrencyTypeEnum
from .request_history import RequestHistory
from .request_history import StatusEnum as RequestHistoryStatusEnum
from .requirement import Requirement
from .revocation_config_info import RevocationConfigInfo
from .revocation_config_update import RevocationConfigUpdate
from .revocation_error import RevocationError
from .revocation_history_info import RevocationHistoryInfo
from .revocation_history_info import StatusEnum as RevocationHistoryInfoStatusEnum
from .revocation_history_paging_sliced_result import RevocationHistoryPagingSlicedResult
from .revocation_request import RevocationRequest
from .revocation_request import SourceEnum as RevocationRequestSourceEnum
from .revocation_result import RevocationResult
from .revocation_result import StatusEnum as RevocationResultStatusEnum
from .revoke_currency import RevokeCurrency
from .revoke_currency import BalanceOriginEnum as RevokeCurrencyBalanceOriginEnum
from .revoke_entitlement import RevokeEntitlement
from .revoke_entry import RevokeEntry
from .revoke_entry import TypeEnum as RevokeEntryTypeEnum
from .revoke_item import RevokeItem
from .revoke_item import (
    ItemIdentityTypeEnum as RevokeItemItemIdentityTypeEnum,
    OriginEnum as RevokeItemOriginEnum,
)
from .revoke_item_summary import RevokeItemSummary
from .revoke_item_summary import RevokeStatusEnum as RevokeItemSummaryRevokeStatusEnum
from .revoke_result import RevokeResult
from .revoke_result import StatusEnum as RevokeResultStatusEnum
from .revoke_use_count_request import RevokeUseCountRequest
from .reward_condition import RewardCondition
from .reward_create import RewardCreate
from .reward_info import RewardInfo
from .reward_item import RewardItem
from .reward_paging_sliced_result import RewardPagingSlicedResult
from .reward_update import RewardUpdate
from .rewards_request import RewardsRequest
from .rewards_request import (
    OriginEnum as RewardsRequestOriginEnum,
    SourceEnum as RewardsRequestSourceEnum,
)
from .sale_config import SaleConfig
from .section_create import SectionCreate
from .section_create import RotationTypeEnum as SectionCreateRotationTypeEnum
from .section_info import SectionInfo
from .section_item import SectionItem
from .section_paging_sliced_result import SectionPagingSlicedResult
from .section_update import SectionUpdate
from .section_update import RotationTypeEnum as SectionUpdateRotationTypeEnum
from .service_plugin_config_info import ServicePluginConfigInfo
from .service_plugin_config_update import ServicePluginConfigUpdate
from .slide import Slide
from .slide import TypeEnum as SlideTypeEnum, VideoSourceEnum as SlideVideoSourceEnum
from .stackable_entitlement_info import StackableEntitlementInfo
from .stackable_entitlement_info import (
    AppTypeEnum as StackableEntitlementInfoAppTypeEnum,
    ClazzEnum as StackableEntitlementInfoClazzEnum,
    SourceEnum as StackableEntitlementInfoSourceEnum,
    StatusEnum as StackableEntitlementInfoStatusEnum,
    TypeEnum as StackableEntitlementInfoTypeEnum,
)
from .steam_achievement_update_request import SteamAchievementUpdateRequest
from .steam_dlc_sync_request import SteamDLCSyncRequest
from .steam_iap_config import SteamIAPConfig
from .steam_iap_config_info import SteamIAPConfigInfo
from .steam_iap_config_request import SteamIAPConfigRequest
from .steam_sync_request import SteamSyncRequest
from .store_backup_info import StoreBackupInfo
from .store_create import StoreCreate
from .store_info import StoreInfo
from .store_update import StoreUpdate
from .stripe_config import StripeConfig
from .subscribable import Subscribable
from .subscribe_request import SubscribeRequest
from .subscription_activity_info import SubscriptionActivityInfo
from .subscription_activity_info import (
    ActionEnum as SubscriptionActivityInfoActionEnum,
    SubscribedByEnum as SubscriptionActivityInfoSubscribedByEnum,
)
from .subscription_activity_paging_sliced_result import (
    SubscriptionActivityPagingSlicedResult,
)
from .subscription_info import SubscriptionInfo
from .subscription_info import (
    ChargeStatusEnum as SubscriptionInfoChargeStatusEnum,
    StatusEnum as SubscriptionInfoStatusEnum,
    SubscribedByEnum as SubscriptionInfoSubscribedByEnum,
)
from .subscription_paging_sliced_result import SubscriptionPagingSlicedResult
from .subscription_summary import SubscriptionSummary
from .subscription_summary import (
    StatusEnum as SubscriptionSummaryStatusEnum,
    SubscribedByEnum as SubscriptionSummarySubscribedByEnum,
)
from .tax_result import TaxResult
from .test_result import TestResult
from .ticket_acquire_request import TicketAcquireRequest
from .ticket_acquire_result import TicketAcquireResult
from .ticket_booth_id import TicketBoothID
from .ticket_booth_id import TypeEnum as TicketBoothIDTypeEnum
from .ticket_dynamic_info import TicketDynamicInfo
from .ticket_sale_decrement_request import TicketSaleDecrementRequest
from .ticket_sale_increment_request import TicketSaleIncrementRequest
from .ticket_sale_increment_result import TicketSaleIncrementResult
from .time_limited_balance import TimeLimitedBalance
from .timed_ownership import TimedOwnership
from .trade_notification import TradeNotification
from .trade_notification import (
    PaymentProviderEnum as TradeNotificationPaymentProviderEnum,
    StatusEnum as TradeNotificationStatusEnum,
)
from .transaction import Transaction
from .transaction import (
    ProviderEnum as TransactionProviderEnum,
    StatusEnum as TransactionStatusEnum,
    TypeEnum as TransactionTypeEnum,
)
from .transaction_amount_details import TransactionAmountDetails
from .twitch_iap_config_info import TwitchIAPConfigInfo
from .twitch_iap_config_request import TwitchIAPConfigRequest
from .twitch_sync_request import TwitchSyncRequest
from .twitch_sync_result import TwitchSyncResult
from .twitch_sync_result import IapOrderStatusEnum as TwitchSyncResultIapOrderStatusEnum
from .user_dlc import UserDLC
from .user_dlc import PlatformEnum as UserDLCPlatformEnum
from .user_dlc_record import UserDLCRecord
from .user_dlc_record import (
    PlatformEnum as UserDLCRecordPlatformEnum,
    StatusEnum as UserDLCRecordStatusEnum,
)
from .validation_error_entity import ValidationErrorEntity
from .view_create import ViewCreate
from .view_info import ViewInfo
from .view_update import ViewUpdate
from .wallet_info import WalletInfo
from .wallet_info import StatusEnum as WalletInfoStatusEnum
from .wallet_paging_sliced_result import WalletPagingSlicedResult
from .wallet_revocation_config import WalletRevocationConfig
from .wallet_revocation_config import StrategyEnum as WalletRevocationConfigStrategyEnum
from .wallet_transaction_info import WalletTransactionInfo
from .wallet_transaction_info import (
    WalletActionEnum as WalletTransactionInfoWalletActionEnum,
)
from .wallet_transaction_paging_sliced_result import WalletTransactionPagingSlicedResult
from .wx_pay_config_info import WxPayConfigInfo
from .wx_pay_config_request import WxPayConfigRequest
from .xbl_achievement_update_request import XblAchievementUpdateRequest
from .xbl_dlc_sync_request import XblDLCSyncRequest
from .xbl_iap_config_info import XblIAPConfigInfo
from .xbl_iap_config_request import XblIAPConfigRequest
from .xbl_reconcile_request import XblReconcileRequest
from .xbl_reconcile_result import XblReconcileResult
from .xbl_reconcile_result import (
    IapOrderStatusEnum as XblReconcileResultIapOrderStatusEnum,
)
from .xbl_user_achievements import XblUserAchievements
from .xbl_user_session_request import XblUserSessionRequest
from .xsolla_config import XsollaConfig
from .xsolla_paywall_config import XsollaPaywallConfig
from .xsolla_paywall_config import (
    DeviceEnum as XsollaPaywallConfigDeviceEnum,
    SizeEnum as XsollaPaywallConfigSizeEnum,
    ThemeEnum as XsollaPaywallConfigThemeEnum,
)
from .xsolla_paywall_config_request import XsollaPaywallConfigRequest
from .xsolla_paywall_config_request import (
    DeviceEnum as XsollaPaywallConfigRequestDeviceEnum,
    SizeEnum as XsollaPaywallConfigRequestSizeEnum,
    ThemeEnum as XsollaPaywallConfigRequestThemeEnum,
)



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

"""Auto-generated package that contains models used by the AccelByte Gaming Services Social Service."""

__version__ = "2.8.0"
__author__ = "AccelByte"
__email__ = "dev@accelbyte.net"

# pylint: disable=line-too-long

from .bulk_add_stats import BulkAddStats
from .bulk_get_stat_cycle import BulkGetStatCycle
from .bulk_get_stat_cycle_1 import BulkGetStatCycle1
from .create_stat_cycle import CreateStatCycle
from .delete_stat_cycle import DeleteStatCycle
from .get_stat_cycle import GetStatCycle
from .get_stat_cycle_1 import GetStatCycle1
from .get_stat_cycles import GetStatCycles
from .get_stat_cycles import (
    CycleTypeEnum as GetStatCyclesCycleTypeEnum,
    StatusEnum as GetStatCyclesStatusEnum,
)
from .get_stat_cycles_1 import GetStatCycles1
from .get_stat_cycles_1 import (
    CycleTypeEnum as GetStatCycles1CycleTypeEnum,
    StatusEnum as GetStatCycles1StatusEnum,
)
from .stop_stat_cycle import StopStatCycle
from .update_stat_cycle import UpdateStatCycle



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Seasonpass Service (1.18.4)

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model

from ..models.tier_input import TierInput


class TierCreate(Model):
    """A DTO object for creating tier API call. (TierCreate)

    Properties:
        index: (index) OPTIONAL int

        quantity: (quantity) OPTIONAL int

        tier: (tier) OPTIONAL TierInput
    """

    # region fields

    index: int  # OPTIONAL
    quantity: int  # OPTIONAL
    tier: TierInput  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_index(self, value: int) -> TierCreate:
        self.index = value
        return self

    def with_quantity(self, value: int) -> TierCreate:
        self.quantity = value
        return self

    def with_tier(self, value: TierInput) -> TierCreate:
        self.tier = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "index"):
            result["index"] = int(self.index)
        elif include_empty:
            result["index"] = 0
        if hasattr(self, "quantity"):
            result["quantity"] = int(self.quantity)
        elif include_empty:
            result["quantity"] = 0
        if hasattr(self, "tier"):
            result["tier"] = self.tier.to_dict(include_empty=include_empty)
        elif include_empty:
            result["tier"] = TierInput()
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        index: Optional[int] = None,
        quantity: Optional[int] = None,
        tier: Optional[TierInput] = None,
        **kwargs,
    ) -> TierCreate:
        instance = cls()
        if index is not None:
            instance.index = index
        if quantity is not None:
            instance.quantity = quantity
        if tier is not None:
            instance.tier = tier
        return instance

    @classmethod
    def create_from_dict(cls, dict_: dict, include_empty: bool = False) -> TierCreate:
        instance = cls()
        if not dict_:
            return instance
        if "index" in dict_ and dict_["index"] is not None:
            instance.index = int(dict_["index"])
        elif include_empty:
            instance.index = 0
        if "quantity" in dict_ and dict_["quantity"] is not None:
            instance.quantity = int(dict_["quantity"])
        elif include_empty:
            instance.quantity = 0
        if "tier" in dict_ and dict_["tier"] is not None:
            instance.tier = TierInput.create_from_dict(
                dict_["tier"], include_empty=include_empty
            )
        elif include_empty:
            instance.tier = TierInput()
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, TierCreate]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[TierCreate]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[TierCreate, List[TierCreate], Dict[Any, TierCreate]]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "index": "index",
            "quantity": "quantity",
            "tier": "tier",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "index": False,
            "quantity": False,
            "tier": False,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Iam Service (6.0.1)

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model

from ..models.model_account_progression_info import ModelAccountProgressionInfo


class ModelGetLinkHeadlessAccountConflictResponse(Model):
    """Model get link headless account conflict response (model.GetLinkHeadlessAccountConflictResponse)

    Properties:
        platform_already_linked: (platformAlreadyLinked) REQUIRED bool

        platform_id: (platformId) REQUIRED str

        platform_link_conflict: (platformLinkConflict) REQUIRED bool

        current_account: (currentAccount) OPTIONAL ModelAccountProgressionInfo

        headless_account: (headlessAccount) OPTIONAL ModelAccountProgressionInfo
    """

    # region fields

    platform_already_linked: bool  # REQUIRED
    platform_id: str  # REQUIRED
    platform_link_conflict: bool  # REQUIRED
    current_account: ModelAccountProgressionInfo  # OPTIONAL
    headless_account: ModelAccountProgressionInfo  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_platform_already_linked(
        self, value: bool
    ) -> ModelGetLinkHeadlessAccountConflictResponse:
        self.platform_already_linked = value
        return self

    def with_platform_id(
        self, value: str
    ) -> ModelGetLinkHeadlessAccountConflictResponse:
        self.platform_id = value
        return self

    def with_platform_link_conflict(
        self, value: bool
    ) -> ModelGetLinkHeadlessAccountConflictResponse:
        self.platform_link_conflict = value
        return self

    def with_current_account(
        self, value: ModelAccountProgressionInfo
    ) -> ModelGetLinkHeadlessAccountConflictResponse:
        self.current_account = value
        return self

    def with_headless_account(
        self, value: ModelAccountProgressionInfo
    ) -> ModelGetLinkHeadlessAccountConflictResponse:
        self.headless_account = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "platform_already_linked"):
            result["platformAlreadyLinked"] = bool(self.platform_already_linked)
        elif include_empty:
            result["platformAlreadyLinked"] = False
        if hasattr(self, "platform_id"):
            result["platformId"] = str(self.platform_id)
        elif include_empty:
            result["platformId"] = ""
        if hasattr(self, "platform_link_conflict"):
            result["platformLinkConflict"] = bool(self.platform_link_conflict)
        elif include_empty:
            result["platformLinkConflict"] = False
        if hasattr(self, "current_account"):
            result["currentAccount"] = self.current_account.to_dict(
                include_empty=include_empty
            )
        elif include_empty:
            result["currentAccount"] = ModelAccountProgressionInfo()
        if hasattr(self, "headless_account"):
            result["headlessAccount"] = self.headless_account.to_dict(
                include_empty=include_empty
            )
        elif include_empty:
            result["headlessAccount"] = ModelAccountProgressionInfo()
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        platform_already_linked: bool,
        platform_id: str,
        platform_link_conflict: bool,
        current_account: Optional[ModelAccountProgressionInfo] = None,
        headless_account: Optional[ModelAccountProgressionInfo] = None,
        **kwargs,
    ) -> ModelGetLinkHeadlessAccountConflictResponse:
        instance = cls()
        instance.platform_already_linked = platform_already_linked
        instance.platform_id = platform_id
        instance.platform_link_conflict = platform_link_conflict
        if current_account is not None:
            instance.current_account = current_account
        if headless_account is not None:
            instance.headless_account = headless_account
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ModelGetLinkHeadlessAccountConflictResponse:
        instance = cls()
        if not dict_:
            return instance
        if (
            "platformAlreadyLinked" in dict_
            and dict_["platformAlreadyLinked"] is not None
        ):
            instance.platform_already_linked = bool(dict_["platformAlreadyLinked"])
        elif include_empty:
            instance.platform_already_linked = False
        if "platformId" in dict_ and dict_["platformId"] is not None:
            instance.platform_id = str(dict_["platformId"])
        elif include_empty:
            instance.platform_id = ""
        if (
            "platformLinkConflict" in dict_
            and dict_["platformLinkConflict"] is not None
        ):
            instance.platform_link_conflict = bool(dict_["platformLinkConflict"])
        elif include_empty:
            instance.platform_link_conflict = False
        if "currentAccount" in dict_ and dict_["currentAccount"] is not None:
            instance.current_account = ModelAccountProgressionInfo.create_from_dict(
                dict_["currentAccount"], include_empty=include_empty
            )
        elif include_empty:
            instance.current_account = ModelAccountProgressionInfo()
        if "headlessAccount" in dict_ and dict_["headlessAccount"] is not None:
            instance.headless_account = ModelAccountProgressionInfo.create_from_dict(
                dict_["headlessAccount"], include_empty=include_empty
            )
        elif include_empty:
            instance.headless_account = ModelAccountProgressionInfo()
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ModelGetLinkHeadlessAccountConflictResponse]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ModelGetLinkHeadlessAccountConflictResponse]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        ModelGetLinkHeadlessAccountConflictResponse,
        List[ModelGetLinkHeadlessAccountConflictResponse],
        Dict[Any, ModelGetLinkHeadlessAccountConflictResponse],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "platformAlreadyLinked": "platform_already_linked",
            "platformId": "platform_id",
            "platformLinkConflict": "platform_link_conflict",
            "currentAccount": "current_account",
            "headlessAccount": "headless_account",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "platformAlreadyLinked": True,
            "platformId": True,
            "platformLinkConflict": True,
            "currentAccount": False,
            "headlessAccount": False,
        }

    # endregion static methods



from datetime import timedelta
from typing import Optional, Protocol


class HttpBackoffPolicy(Protocol):
    def __call__(
        self,
        request,
        response,
        /,
        *,
        retries: int = 0,
        elapsed: Optional[timedelta] = None,
        **kwargs,
    ) -> float:
        ...


class ConstantHttpBackoffPolicy:
    def __init__(self, duration: float):
        self.duration = duration

    def __call__(
        self,
        request,
        response,
        /,
        *,
        retries: int = 0,
        elapsed: Optional[timedelta] = None,
        **kwargs,
    ) -> float:
        return self.duration


class ExponentialHttpBackoffPolicy:
    def __init__(self, initial: float = 1.0, multiplier: float = 2.0):
        if initial == 0.0:
            initial = 0.01
        self.initial = initial
        self.multiplier = multiplier

    def __call__(
        self,
        request,
        response,
        /,
        *,
        retries: int = 0,
        elapsed: Optional[timedelta] = None,
        **kwargs,
    ) -> float:
        return self.initial * (self.multiplier**retries)


class NoHttpBackoffPolicy(ConstantHttpBackoffPolicy):
    def __init__(self):
        super().__init__(0.0)

    def __call__(
        self,
        request,
        response,
        /,
        *,
        retries: int = 0,
        elapsed: Optional[timedelta] = None,
        **kwargs,
    ) -> float:
        return self.duration



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import HeaderStr
from ....core import get_namespace as get_services_namespace
from ....core import run_request
from ....core import run_request_async
from ....core import same_doc_as

from ..models import ApiListMatchFunctionsResponse
from ..models import ApiMatchFunctionConfig
from ..models import ApiMatchFunctionRequest
from ..models import ResponseError

from ..operations.match_functions import CreateMatchFunction
from ..operations.match_functions import DeleteMatchFunction
from ..operations.match_functions import MatchFunctionList
from ..operations.match_functions import UpdateMatchFunction


@same_doc_as(CreateMatchFunction)
def create_match_function(
    body: ApiMatchFunctionRequest,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Create a match function (CreateMatchFunction)

    Required Permission: NAMESPACE:{namespace}:MATCHMAKING:FUNCTIONS [CREATE]

    Required Scope: social

    Creates a new matchmaking function.

    Required Permission(s):
        - NAMESPACE:{namespace}:MATCHMAKING:FUNCTIONS [CREATE]

    Required Scope(s):
        - social

    Properties:
        url: /match2/v1/namespaces/{namespace}/match-functions

        method: POST

        tags: ["Match-Functions", "admin"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ApiMatchFunctionRequest in body

        namespace: (namespace) REQUIRED str in path

    Responses:
        201: Created - (Created)

        400: Bad Request - ResponseError (Bad Request)

        401: Unauthorized - ResponseError (Unauthorized)

        403: Forbidden - ResponseError (Forbidden)

        409: Conflict - ResponseError (Conflict)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = CreateMatchFunction.create(
        body=body,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(CreateMatchFunction)
async def create_match_function_async(
    body: ApiMatchFunctionRequest,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Create a match function (CreateMatchFunction)

    Required Permission: NAMESPACE:{namespace}:MATCHMAKING:FUNCTIONS [CREATE]

    Required Scope: social

    Creates a new matchmaking function.

    Required Permission(s):
        - NAMESPACE:{namespace}:MATCHMAKING:FUNCTIONS [CREATE]

    Required Scope(s):
        - social

    Properties:
        url: /match2/v1/namespaces/{namespace}/match-functions

        method: POST

        tags: ["Match-Functions", "admin"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ApiMatchFunctionRequest in body

        namespace: (namespace) REQUIRED str in path

    Responses:
        201: Created - (Created)

        400: Bad Request - ResponseError (Bad Request)

        401: Unauthorized - ResponseError (Unauthorized)

        403: Forbidden - ResponseError (Forbidden)

        409: Conflict - ResponseError (Conflict)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = CreateMatchFunction.create(
        body=body,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(DeleteMatchFunction)
def delete_match_function(
    name: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Delete a match function (DeleteMatchFunction)

    Required Permission: NAMESPACE:{namespace}:MATCHMAKING:FUNCTIONS [DELETE]

    Required Scope: social

    Deletes an existing match function.

    Required Permission(s):
        - NAMESPACE:{namespace}:MATCHMAKING:FUNCTIONS [DELETE]

    Required Scope(s):
        - social

    Properties:
        url: /match2/v1/namespaces/{namespace}/match-functions/{name}

        method: DELETE

        tags: ["Match-Functions", "admin"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        name: (name) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - (OK)

        401: Unauthorized - ResponseError (Unauthorized)

        403: Forbidden - ResponseError (Forbidden)

        404: Not Found - ResponseError (Not Found)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = DeleteMatchFunction.create(
        name=name,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(DeleteMatchFunction)
async def delete_match_function_async(
    name: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Delete a match function (DeleteMatchFunction)

    Required Permission: NAMESPACE:{namespace}:MATCHMAKING:FUNCTIONS [DELETE]

    Required Scope: social

    Deletes an existing match function.

    Required Permission(s):
        - NAMESPACE:{namespace}:MATCHMAKING:FUNCTIONS [DELETE]

    Required Scope(s):
        - social

    Properties:
        url: /match2/v1/namespaces/{namespace}/match-functions/{name}

        method: DELETE

        tags: ["Match-Functions", "admin"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        name: (name) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - (OK)

        401: Unauthorized - ResponseError (Unauthorized)

        403: Forbidden - ResponseError (Forbidden)

        404: Not Found - ResponseError (Not Found)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = DeleteMatchFunction.create(
        name=name,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(MatchFunctionList)
def match_function_list(
    limit: Optional[int] = None,
    offset: Optional[int] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """List existing match functions (MatchFunctionList)

    Required Permission: NAMESPACE:{namespace}:MATCHMAKING:FUNCTIONS [READ]

    Required Scope: social

    List existing match functions.

    Required Permission(s):
        - NAMESPACE:{namespace}:MATCHMAKING:FUNCTIONS [READ]

    Required Scope(s):
        - social

    Properties:
        url: /match2/v1/namespaces/{namespace}/match-functions

        method: GET

        tags: ["Match-Functions", "admin"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        limit: (limit) OPTIONAL int in query

        offset: (offset) OPTIONAL int in query

    Responses:
        200: OK - ApiListMatchFunctionsResponse (Created)

        401: Unauthorized - ResponseError (Unauthorized)

        403: Forbidden - ResponseError (Forbidden)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = MatchFunctionList.create(
        limit=limit,
        offset=offset,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(MatchFunctionList)
async def match_function_list_async(
    limit: Optional[int] = None,
    offset: Optional[int] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """List existing match functions (MatchFunctionList)

    Required Permission: NAMESPACE:{namespace}:MATCHMAKING:FUNCTIONS [READ]

    Required Scope: social

    List existing match functions.

    Required Permission(s):
        - NAMESPACE:{namespace}:MATCHMAKING:FUNCTIONS [READ]

    Required Scope(s):
        - social

    Properties:
        url: /match2/v1/namespaces/{namespace}/match-functions

        method: GET

        tags: ["Match-Functions", "admin"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        limit: (limit) OPTIONAL int in query

        offset: (offset) OPTIONAL int in query

    Responses:
        200: OK - ApiListMatchFunctionsResponse (Created)

        401: Unauthorized - ResponseError (Unauthorized)

        403: Forbidden - ResponseError (Forbidden)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = MatchFunctionList.create(
        limit=limit,
        offset=offset,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(UpdateMatchFunction)
def update_match_function(
    body: ApiMatchFunctionRequest,
    name: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Update a match function (UpdateMatchFunction)

    Required Permission: NAMESPACE:{namespace}:MATCHMAKING:FUNCTIONS [UPDATE]

    Required Scope: social

    Update existing matchmaking function.

    Required Permission(s):
        - NAMESPACE:{namespace}:MATCHMAKING:FUNCTIONS [UPDATE]

    Required Scope(s):
        - social

    Properties:
        url: /match2/v1/namespaces/{namespace}/match-functions/{name}

        method: PUT

        tags: ["Match-Functions", "admin"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ApiMatchFunctionRequest in body

        name: (name) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ApiMatchFunctionConfig (OK)

        400: Bad Request - ResponseError (Bad Request)

        401: Unauthorized - ResponseError (Unauthorized)

        404: Not Found - ResponseError (Not Found)

        409: Conflict - ResponseError (Conflict)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = UpdateMatchFunction.create(
        body=body,
        name=name,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(UpdateMatchFunction)
async def update_match_function_async(
    body: ApiMatchFunctionRequest,
    name: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Update a match function (UpdateMatchFunction)

    Required Permission: NAMESPACE:{namespace}:MATCHMAKING:FUNCTIONS [UPDATE]

    Required Scope: social

    Update existing matchmaking function.

    Required Permission(s):
        - NAMESPACE:{namespace}:MATCHMAKING:FUNCTIONS [UPDATE]

    Required Scope(s):
        - social

    Properties:
        url: /match2/v1/namespaces/{namespace}/match-functions/{name}

        method: PUT

        tags: ["Match-Functions", "admin"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ApiMatchFunctionRequest in body

        name: (name) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ApiMatchFunctionConfig (OK)

        400: Bad Request - ResponseError (Bad Request)

        401: Unauthorized - ResponseError (Unauthorized)

        404: Not Found - ResponseError (Not Found)

        409: Conflict - ResponseError (Conflict)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = UpdateMatchFunction.create(
        body=body,
        name=name,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )



from typing import Optional, Protocol

from ._ctypes import FetchValidationDataError
from ._ctypes import FetchRoleError
from ._ctypes import TokenValidationError
from ._ctypes import InsufficientPermissionsError
from ._ctypes import TokenRevokedError
from ._ctypes import UserRevokedError

from ._ctypes import PermissionAction
from ._ctypes import PermissionStruct
from ._ctypes import create_permission_struct

from ._utils import replace_resource
from ._utils import validate_permission


class TokenValidatorProtocol(Protocol):
    def validate_token(
        self,
        token: str,
        resource: Optional[str] = None,
        action: Optional[PermissionAction] = None,
        namespace: Optional[str] = None,
        **kwargs,
    ) -> Optional[Exception]:
        ...


__all__ = [
    "TokenValidatorProtocol",
]



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Matchmaking Service (2.25.1)

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model


class ModelsStatResumeResponse(Model):
    """Models stat resume response (models.StatResumeResponse)

    Properties:
        max: (max) REQUIRED str

        mean: (mean) REQUIRED str

        min: (min) REQUIRED str

        multiplier: (multiplier) REQUIRED str

        stddev: (stddev) REQUIRED str

        value_shifts: (valueShifts) REQUIRED Dict[str, str]

        zscores: (zscores) REQUIRED Dict[str, str]
    """

    # region fields

    max: str  # REQUIRED
    mean: str  # REQUIRED
    min: str  # REQUIRED
    multiplier: str  # REQUIRED
    stddev: str  # REQUIRED
    value_shifts: Dict[str, str]  # REQUIRED
    zscores: Dict[str, str]  # REQUIRED

    # endregion fields

    # region with_x methods

    def with_max(self, value: str) -> ModelsStatResumeResponse:
        self.max = value
        return self

    def with_mean(self, value: str) -> ModelsStatResumeResponse:
        self.mean = value
        return self

    def with_min(self, value: str) -> ModelsStatResumeResponse:
        self.min = value
        return self

    def with_multiplier(self, value: str) -> ModelsStatResumeResponse:
        self.multiplier = value
        return self

    def with_stddev(self, value: str) -> ModelsStatResumeResponse:
        self.stddev = value
        return self

    def with_value_shifts(self, value: Dict[str, str]) -> ModelsStatResumeResponse:
        self.value_shifts = value
        return self

    def with_zscores(self, value: Dict[str, str]) -> ModelsStatResumeResponse:
        self.zscores = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "max"):
            result["max"] = str(self.max)
        elif include_empty:
            result["max"] = ""
        if hasattr(self, "mean"):
            result["mean"] = str(self.mean)
        elif include_empty:
            result["mean"] = ""
        if hasattr(self, "min"):
            result["min"] = str(self.min)
        elif include_empty:
            result["min"] = ""
        if hasattr(self, "multiplier"):
            result["multiplier"] = str(self.multiplier)
        elif include_empty:
            result["multiplier"] = ""
        if hasattr(self, "stddev"):
            result["stddev"] = str(self.stddev)
        elif include_empty:
            result["stddev"] = ""
        if hasattr(self, "value_shifts"):
            result["valueShifts"] = {
                str(k0): str(v0) for k0, v0 in self.value_shifts.items()
            }
        elif include_empty:
            result["valueShifts"] = {}
        if hasattr(self, "zscores"):
            result["zscores"] = {str(k0): str(v0) for k0, v0 in self.zscores.items()}
        elif include_empty:
            result["zscores"] = {}
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        max: str,
        mean: str,
        min: str,
        multiplier: str,
        stddev: str,
        value_shifts: Dict[str, str],
        zscores: Dict[str, str],
        **kwargs,
    ) -> ModelsStatResumeResponse:
        instance = cls()
        instance.max = max
        instance.mean = mean
        instance.min = min
        instance.multiplier = multiplier
        instance.stddev = stddev
        instance.value_shifts = value_shifts
        instance.zscores = zscores
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ModelsStatResumeResponse:
        instance = cls()
        if not dict_:
            return instance
        if "max" in dict_ and dict_["max"] is not None:
            instance.max = str(dict_["max"])
        elif include_empty:
            instance.max = ""
        if "mean" in dict_ and dict_["mean"] is not None:
            instance.mean = str(dict_["mean"])
        elif include_empty:
            instance.mean = ""
        if "min" in dict_ and dict_["min"] is not None:
            instance.min = str(dict_["min"])
        elif include_empty:
            instance.min = ""
        if "multiplier" in dict_ and dict_["multiplier"] is not None:
            instance.multiplier = str(dict_["multiplier"])
        elif include_empty:
            instance.multiplier = ""
        if "stddev" in dict_ and dict_["stddev"] is not None:
            instance.stddev = str(dict_["stddev"])
        elif include_empty:
            instance.stddev = ""
        if "valueShifts" in dict_ and dict_["valueShifts"] is not None:
            instance.value_shifts = {
                str(k0): str(v0) for k0, v0 in dict_["valueShifts"].items()
            }
        elif include_empty:
            instance.value_shifts = {}
        if "zscores" in dict_ and dict_["zscores"] is not None:
            instance.zscores = {str(k0): str(v0) for k0, v0 in dict_["zscores"].items()}
        elif include_empty:
            instance.zscores = {}
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ModelsStatResumeResponse]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ModelsStatResumeResponse]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        ModelsStatResumeResponse,
        List[ModelsStatResumeResponse],
        Dict[Any, ModelsStatResumeResponse],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "max": "max",
            "mean": "mean",
            "min": "min",
            "multiplier": "multiplier",
            "stddev": "stddev",
            "valueShifts": "value_shifts",
            "zscores": "zscores",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "max": True,
            "mean": True,
            "min": True,
            "multiplier": True,
            "stddev": True,
            "valueShifts": True,
            "zscores": True,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

# AccelByte Gaming Services Iam Service (6.0.1)

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from .....core import Operation
from .....core import HeaderStr
from .....core import HttpResponse

from ...models import ModelInputValidationUpdatePayload
from ...models import RestErrorResponse


class AdminUpdateInputValidations(Operation):
    """Admin Update Input Validations (AdminUpdateInputValidations)

    Required permission 'ADMIN:CONFIGURATION' [UPDATE]




    This endpoint is used to update input validation configuration.


    Supported `field`:





      * displayName


      * password


      * username


      * email


      * avatar





    If `isCustomRegex` is set to true, `regex` parameter will be used as input validation and the other parameters will be ignored. Otherwise, `regex` parameter will be ignored and regex for input validation will be generated based on the combination of the other parameters.




    If `allowUnicode` is set to true, unicode regex pattern will be use as the input validation and the other parameters will be ignored.


    Supported `letterCase`:





      * lowercase


      * uppercase


      * mixed: uppercase and lowercase


      * mixed: uppercase and/or lowercase



    flexible special character non words with `allowAllSpecialCharacters`



    if `allowAllSpecialCharacters` is set to true `specialCharacters` will forced to empty.


    Supported `specialCharacterLocation`:





      * anywhere


      * middle





    If `specialCharacters` is empty, `specialCharacterLocation` and `maxRepeatingSpecialCharacter` will be ignored.




    `minCharType` is used to identify how many required criteria in the regex. The supported criteria are number, letter, special character, and letter case. If set to 0 or 1 means all criteria are optional. It can be set as much as the number of criteria enabled.




    If `blockedWord` is set by admin, any input from user which contain kind of blocked word(s) will be blocked for create/upgrade/update account




    If `avatarConfig` is set, will use this config and skip all the other validation conditions

    Required Permission(s):
        - ADMIN:CONFIGURATION [UPDATE]

    Properties:
        url: /iam/v3/admin/inputValidations

        method: PUT

        tags: ["InputValidations"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED List[ModelInputValidationUpdatePayload] in body

    Responses:
        204: No Content - (No Content)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        404: Not Found - RestErrorResponse
    """

    # region fields

    _url: str = "/iam/v3/admin/inputValidations"
    _method: str = "PUT"
    _consumes: List[str] = ["application/json"]
    _produces: List[str] = ["application/json"]
    _securities: List[List[str]] = [["BEARER_AUTH"]]
    _location_query: str = None

    body: List[ModelInputValidationUpdatePayload]  # REQUIRED in [body]

    # endregion fields

    # region properties

    @property
    def url(self) -> str:
        return self._url

    @property
    def method(self) -> str:
        return self._method

    @property
    def consumes(self) -> List[str]:
        return self._consumes

    @property
    def produces(self) -> List[str]:
        return self._produces

    @property
    def securities(self) -> List[List[str]]:
        return self._securities

    @property
    def location_query(self) -> str:
        return self._location_query

    # endregion properties

    # region get methods

    # endregion get methods

    # region get_x_params methods

    def get_all_params(self) -> dict:
        return {
            "body": self.get_body_params(),
        }

    def get_body_params(self) -> Any:
        if not hasattr(self, "body") or self.body is None:
            return None
        return [i.to_dict() for i in self.body]

    # endregion get_x_params methods

    # region is/has methods

    # endregion is/has methods

    # region with_x methods

    def with_body(
        self, value: List[ModelInputValidationUpdatePayload]
    ) -> AdminUpdateInputValidations:
        self.body = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "body") and self.body:
            result["body"] = [
                i0.to_dict(include_empty=include_empty) for i0 in self.body
            ]
        elif include_empty:
            result["body"] = []
        return result

    # endregion to methods

    # region response methods

    # noinspection PyMethodMayBeStatic
    def parse_response(
        self, code: int, content_type: str, content: Any
    ) -> Tuple[None, Union[None, HttpResponse, RestErrorResponse]]:
        """Parse the given response.

        204: No Content - (No Content)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        404: Not Found - RestErrorResponse

        ---: HttpResponse (Undocumented Response)

        ---: HttpResponse (Unexpected Content-Type Error)

        ---: HttpResponse (Unhandled Error)
        """
        pre_processed_response, error = self.pre_process_response(
            code=code, content_type=content_type, content=content
        )
        if error is not None:
            return None, None if error.is_no_content() else error
        code, content_type, content = pre_processed_response

        if code == 204:
            return None, None
        if code == 401:
            return None, RestErrorResponse.create_from_dict(content)
        if code == 403:
            return None, RestErrorResponse.create_from_dict(content)
        if code == 404:
            return None, RestErrorResponse.create_from_dict(content)

        return self.handle_undocumented_response(
            code=code, content_type=content_type, content=content
        )

    # endregion response methods

    # region static methods

    @classmethod
    def create(
        cls, body: List[ModelInputValidationUpdatePayload], **kwargs
    ) -> AdminUpdateInputValidations:
        instance = cls()
        instance.body = body
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> AdminUpdateInputValidations:
        instance = cls()
        if "body" in dict_ and dict_["body"] is not None:
            instance.body = [
                ModelInputValidationUpdatePayload.create_from_dict(
                    i0, include_empty=include_empty
                )
                for i0 in dict_["body"]
            ]
        elif include_empty:
            instance.body = []
        return instance

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "body": "body",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "body": True,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import HeaderStr
from ....core import get_namespace as get_services_namespace
from ....core import run_request
from ....core import run_request_async
from ....core import same_doc_as

from ..models import ADTOForUpdateEqu8ConfigAPICall
from ..models import Equ8Config
from ..models import ErrorEntity

from ..operations.equ8_config import DeleteConfig
from ..operations.equ8_config import GetConfig
from ..operations.equ8_config import UpdateConfig


@same_doc_as(DeleteConfig)
def delete_config(
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Delete equ8 config (deleteConfig)

    Delete equ8 config.
    Other detail info:

      * Required permission : resource= "ADMIN:NAMESPACE:{namespace}:EQU8CONFIG" , action=8 (DELETE)

    Properties:
        url: /basic/v1/admin/namespaces/{namespace}/equ8/config

        method: DELETE

        tags: ["EQU8Config"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

    Responses:
        204: No Content - (Delete equ8 config successfully)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = DeleteConfig.create(
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(DeleteConfig)
async def delete_config_async(
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Delete equ8 config (deleteConfig)

    Delete equ8 config.
    Other detail info:

      * Required permission : resource= "ADMIN:NAMESPACE:{namespace}:EQU8CONFIG" , action=8 (DELETE)

    Properties:
        url: /basic/v1/admin/namespaces/{namespace}/equ8/config

        method: DELETE

        tags: ["EQU8Config"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

    Responses:
        204: No Content - (Delete equ8 config successfully)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = DeleteConfig.create(
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(GetConfig)
def get_config(
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get equ8 config (getConfig)

    Get equ8 config.
    Other detail info:

      * Required permission : resource= "ADMIN:NAMESPACE:{namespace}:EQU8CONFIG" , action=2 (READ)

    Properties:
        url: /basic/v1/admin/namespaces/{namespace}/equ8/config

        method: GET

        tags: ["EQU8Config"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - Equ8Config (successful operation)

        404: Not Found - ErrorEntity (11641: Equ8 config not found in namespace [{namespace}])

        500: Internal Server Error - ErrorEntity (20000: internal server error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetConfig.create(
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(GetConfig)
async def get_config_async(
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get equ8 config (getConfig)

    Get equ8 config.
    Other detail info:

      * Required permission : resource= "ADMIN:NAMESPACE:{namespace}:EQU8CONFIG" , action=2 (READ)

    Properties:
        url: /basic/v1/admin/namespaces/{namespace}/equ8/config

        method: GET

        tags: ["EQU8Config"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - Equ8Config (successful operation)

        404: Not Found - ErrorEntity (11641: Equ8 config not found in namespace [{namespace}])

        500: Internal Server Error - ErrorEntity (20000: internal server error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetConfig.create(
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(UpdateConfig)
def update_config(
    body: Optional[ADTOForUpdateEqu8ConfigAPICall] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Create or Update equ8 config (updateConfig)

    Update equ8 config, create if not exists.
    Other detail info:

      * Required permission : resource= "ADMIN:NAMESPACE:{namespace}:EQU8CONFIG" , action=4 (UPDATE)

    Properties:
        url: /basic/v1/admin/namespaces/{namespace}/equ8/config

        method: PATCH

        tags: ["EQU8Config"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) OPTIONAL ADTOForUpdateEqu8ConfigAPICall in body

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - Equ8Config (successful operation)

        400: Bad Request - ErrorEntity (11621: Invalid EQU8 api key in namespace [{namespace}])

        500: Internal Server Error - ErrorEntity (20000: internal server error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = UpdateConfig.create(
        body=body,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(UpdateConfig)
async def update_config_async(
    body: Optional[ADTOForUpdateEqu8ConfigAPICall] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Create or Update equ8 config (updateConfig)

    Update equ8 config, create if not exists.
    Other detail info:

      * Required permission : resource= "ADMIN:NAMESPACE:{namespace}:EQU8CONFIG" , action=4 (UPDATE)

    Properties:
        url: /basic/v1/admin/namespaces/{namespace}/equ8/config

        method: PATCH

        tags: ["EQU8Config"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) OPTIONAL ADTOForUpdateEqu8ConfigAPICall in body

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - Equ8Config (successful operation)

        400: Bad Request - ErrorEntity (11621: Invalid EQU8 api key in namespace [{namespace}])

        500: Internal Server Error - ErrorEntity (20000: internal server error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = UpdateConfig.create(
        body=body,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Platform Service (4.30.2)

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model
from ....core import StrEnum

from ..models.revoke_entry import RevokeEntry


class SourceEnum(StrEnum):
    DLC = "DLC"
    ORDER = "ORDER"
    OTHER = "OTHER"


class RevocationRequest(Model):
    """Revocation request (RevocationRequest)

    Properties:
        meta: (meta) OPTIONAL Dict[str, Any]

        revoke_entries: (revokeEntries) OPTIONAL List[RevokeEntry]

        source: (source) OPTIONAL Union[str, SourceEnum]

        transaction_id: (transactionId) OPTIONAL str
    """

    # region fields

    meta: Dict[str, Any]  # OPTIONAL
    revoke_entries: List[RevokeEntry]  # OPTIONAL
    source: Union[str, SourceEnum]  # OPTIONAL
    transaction_id: str  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_meta(self, value: Dict[str, Any]) -> RevocationRequest:
        self.meta = value
        return self

    def with_revoke_entries(self, value: List[RevokeEntry]) -> RevocationRequest:
        self.revoke_entries = value
        return self

    def with_source(self, value: Union[str, SourceEnum]) -> RevocationRequest:
        self.source = value
        return self

    def with_transaction_id(self, value: str) -> RevocationRequest:
        self.transaction_id = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "meta"):
            result["meta"] = {str(k0): v0 for k0, v0 in self.meta.items()}
        elif include_empty:
            result["meta"] = {}
        if hasattr(self, "revoke_entries"):
            result["revokeEntries"] = [
                i0.to_dict(include_empty=include_empty) for i0 in self.revoke_entries
            ]
        elif include_empty:
            result["revokeEntries"] = []
        if hasattr(self, "source"):
            result["source"] = str(self.source)
        elif include_empty:
            result["source"] = Union[str, SourceEnum]()
        if hasattr(self, "transaction_id"):
            result["transactionId"] = str(self.transaction_id)
        elif include_empty:
            result["transactionId"] = ""
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        meta: Optional[Dict[str, Any]] = None,
        revoke_entries: Optional[List[RevokeEntry]] = None,
        source: Optional[Union[str, SourceEnum]] = None,
        transaction_id: Optional[str] = None,
        **kwargs,
    ) -> RevocationRequest:
        instance = cls()
        if meta is not None:
            instance.meta = meta
        if revoke_entries is not None:
            instance.revoke_entries = revoke_entries
        if source is not None:
            instance.source = source
        if transaction_id is not None:
            instance.transaction_id = transaction_id
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> RevocationRequest:
        instance = cls()
        if not dict_:
            return instance
        if "meta" in dict_ and dict_["meta"] is not None:
            instance.meta = {str(k0): v0 for k0, v0 in dict_["meta"].items()}
        elif include_empty:
            instance.meta = {}
        if "revokeEntries" in dict_ and dict_["revokeEntries"] is not None:
            instance.revoke_entries = [
                RevokeEntry.create_from_dict(i0, include_empty=include_empty)
                for i0 in dict_["revokeEntries"]
            ]
        elif include_empty:
            instance.revoke_entries = []
        if "source" in dict_ and dict_["source"] is not None:
            instance.source = str(dict_["source"])
        elif include_empty:
            instance.source = Union[str, SourceEnum]()
        if "transactionId" in dict_ and dict_["transactionId"] is not None:
            instance.transaction_id = str(dict_["transactionId"])
        elif include_empty:
            instance.transaction_id = ""
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, RevocationRequest]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[RevocationRequest]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        RevocationRequest, List[RevocationRequest], Dict[Any, RevocationRequest]
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "meta": "meta",
            "revokeEntries": "revoke_entries",
            "source": "source",
            "transactionId": "transaction_id",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "meta": False,
            "revokeEntries": False,
            "source": False,
            "transactionId": False,
        }

    @staticmethod
    def get_enum_map() -> Dict[str, List[Any]]:
        return {
            "source": ["DLC", "ORDER", "OTHER"],
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# Fleet Command (0.1.0)

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model


class ApiFleetServerHistoryEventResponse(Model):
    """Api fleet server history event response (api.FleetServerHistoryEventResponse)

    Properties:
        created_at: (createdAt) REQUIRED str

        exit_code: (exitCode) REQUIRED int

        fleet_id: (fleetId) REQUIRED str

        new_state: (newState) REQUIRED str

        old_state: (oldState) REQUIRED str

        reason: (reason) REQUIRED str

        server_id: (serverId) REQUIRED str
    """

    # region fields

    created_at: str  # REQUIRED
    exit_code: int  # REQUIRED
    fleet_id: str  # REQUIRED
    new_state: str  # REQUIRED
    old_state: str  # REQUIRED
    reason: str  # REQUIRED
    server_id: str  # REQUIRED

    # endregion fields

    # region with_x methods

    def with_created_at(self, value: str) -> ApiFleetServerHistoryEventResponse:
        self.created_at = value
        return self

    def with_exit_code(self, value: int) -> ApiFleetServerHistoryEventResponse:
        self.exit_code = value
        return self

    def with_fleet_id(self, value: str) -> ApiFleetServerHistoryEventResponse:
        self.fleet_id = value
        return self

    def with_new_state(self, value: str) -> ApiFleetServerHistoryEventResponse:
        self.new_state = value
        return self

    def with_old_state(self, value: str) -> ApiFleetServerHistoryEventResponse:
        self.old_state = value
        return self

    def with_reason(self, value: str) -> ApiFleetServerHistoryEventResponse:
        self.reason = value
        return self

    def with_server_id(self, value: str) -> ApiFleetServerHistoryEventResponse:
        self.server_id = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "created_at"):
            result["createdAt"] = str(self.created_at)
        elif include_empty:
            result["createdAt"] = ""
        if hasattr(self, "exit_code"):
            result["exitCode"] = int(self.exit_code)
        elif include_empty:
            result["exitCode"] = 0
        if hasattr(self, "fleet_id"):
            result["fleetId"] = str(self.fleet_id)
        elif include_empty:
            result["fleetId"] = ""
        if hasattr(self, "new_state"):
            result["newState"] = str(self.new_state)
        elif include_empty:
            result["newState"] = ""
        if hasattr(self, "old_state"):
            result["oldState"] = str(self.old_state)
        elif include_empty:
            result["oldState"] = ""
        if hasattr(self, "reason"):
            result["reason"] = str(self.reason)
        elif include_empty:
            result["reason"] = ""
        if hasattr(self, "server_id"):
            result["serverId"] = str(self.server_id)
        elif include_empty:
            result["serverId"] = ""
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        created_at: str,
        exit_code: int,
        fleet_id: str,
        new_state: str,
        old_state: str,
        reason: str,
        server_id: str,
        **kwargs,
    ) -> ApiFleetServerHistoryEventResponse:
        instance = cls()
        instance.created_at = created_at
        instance.exit_code = exit_code
        instance.fleet_id = fleet_id
        instance.new_state = new_state
        instance.old_state = old_state
        instance.reason = reason
        instance.server_id = server_id
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ApiFleetServerHistoryEventResponse:
        instance = cls()
        if not dict_:
            return instance
        if "createdAt" in dict_ and dict_["createdAt"] is not None:
            instance.created_at = str(dict_["createdAt"])
        elif include_empty:
            instance.created_at = ""
        if "exitCode" in dict_ and dict_["exitCode"] is not None:
            instance.exit_code = int(dict_["exitCode"])
        elif include_empty:
            instance.exit_code = 0
        if "fleetId" in dict_ and dict_["fleetId"] is not None:
            instance.fleet_id = str(dict_["fleetId"])
        elif include_empty:
            instance.fleet_id = ""
        if "newState" in dict_ and dict_["newState"] is not None:
            instance.new_state = str(dict_["newState"])
        elif include_empty:
            instance.new_state = ""
        if "oldState" in dict_ and dict_["oldState"] is not None:
            instance.old_state = str(dict_["oldState"])
        elif include_empty:
            instance.old_state = ""
        if "reason" in dict_ and dict_["reason"] is not None:
            instance.reason = str(dict_["reason"])
        elif include_empty:
            instance.reason = ""
        if "serverId" in dict_ and dict_["serverId"] is not None:
            instance.server_id = str(dict_["serverId"])
        elif include_empty:
            instance.server_id = ""
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ApiFleetServerHistoryEventResponse]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ApiFleetServerHistoryEventResponse]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        ApiFleetServerHistoryEventResponse,
        List[ApiFleetServerHistoryEventResponse],
        Dict[Any, ApiFleetServerHistoryEventResponse],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "createdAt": "created_at",
            "exitCode": "exit_code",
            "fleetId": "fleet_id",
            "newState": "new_state",
            "oldState": "old_state",
            "reason": "reason",
            "serverId": "server_id",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "createdAt": True,
            "exitCode": True,
            "fleetId": True,
            "newState": True,
            "oldState": True,
            "reason": True,
            "serverId": True,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Seasonpass Service (1.18.4)

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model


class UserPurchasable(Model):
    """A DTO object for user purchasable. (UserPurchasable)

    Properties:
        pass_item_id: (passItemId) OPTIONAL str

        tier_item_count: (tierItemCount) OPTIONAL int

        tier_item_id: (tierItemId) OPTIONAL str
    """

    # region fields

    pass_item_id: str  # OPTIONAL
    tier_item_count: int  # OPTIONAL
    tier_item_id: str  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_pass_item_id(self, value: str) -> UserPurchasable:
        self.pass_item_id = value
        return self

    def with_tier_item_count(self, value: int) -> UserPurchasable:
        self.tier_item_count = value
        return self

    def with_tier_item_id(self, value: str) -> UserPurchasable:
        self.tier_item_id = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "pass_item_id"):
            result["passItemId"] = str(self.pass_item_id)
        elif include_empty:
            result["passItemId"] = ""
        if hasattr(self, "tier_item_count"):
            result["tierItemCount"] = int(self.tier_item_count)
        elif include_empty:
            result["tierItemCount"] = 0
        if hasattr(self, "tier_item_id"):
            result["tierItemId"] = str(self.tier_item_id)
        elif include_empty:
            result["tierItemId"] = ""
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        pass_item_id: Optional[str] = None,
        tier_item_count: Optional[int] = None,
        tier_item_id: Optional[str] = None,
        **kwargs,
    ) -> UserPurchasable:
        instance = cls()
        if pass_item_id is not None:
            instance.pass_item_id = pass_item_id
        if tier_item_count is not None:
            instance.tier_item_count = tier_item_count
        if tier_item_id is not None:
            instance.tier_item_id = tier_item_id
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> UserPurchasable:
        instance = cls()
        if not dict_:
            return instance
        if "passItemId" in dict_ and dict_["passItemId"] is not None:
            instance.pass_item_id = str(dict_["passItemId"])
        elif include_empty:
            instance.pass_item_id = ""
        if "tierItemCount" in dict_ and dict_["tierItemCount"] is not None:
            instance.tier_item_count = int(dict_["tierItemCount"])
        elif include_empty:
            instance.tier_item_count = 0
        if "tierItemId" in dict_ and dict_["tierItemId"] is not None:
            instance.tier_item_id = str(dict_["tierItemId"])
        elif include_empty:
            instance.tier_item_id = ""
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, UserPurchasable]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[UserPurchasable]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[UserPurchasable, List[UserPurchasable], Dict[Any, UserPurchasable]]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "passItemId": "pass_item_id",
            "tierItemCount": "tier_item_count",
            "tierItemId": "tier_item_id",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "passItemId": False,
            "tierItemCount": False,
            "tierItemId": False,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import HeaderStr
from ....core import get_namespace as get_services_namespace
from ....core import run_request
from ....core import run_request_async
from ....core import same_doc_as

from ..models import ErrorEntity
from ..models import FullSectionInfo
from ..models import SectionCreate
from ..models import SectionInfo
from ..models import SectionPagingSlicedResult
from ..models import SectionUpdate
from ..models import ValidationErrorEntity

from ..operations.section import CreateSection
from ..operations.section import DeleteSection
from ..operations.section import GetSection
from ..operations.section import PublicListActiveSections
from ..operations.section import PurgeExpiredSection
from ..operations.section import QuerySections
from ..operations.section import UpdateSection
from ..models import FullSectionInfoRotationTypeEnum
from ..models import SectionCreateRotationTypeEnum
from ..models import SectionUpdateRotationTypeEnum


@same_doc_as(CreateSection)
def create_section(
    store_id: str,
    body: Optional[SectionCreate] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Create a section (createSection)

    This API is used to create a section.

    Other detail info:

      * Required permission : resource="ADMIN:NAMESPACE:{namespace}:STORE", action=1 (CREATE)
      *  Returns : created a section



    ## Restrictions for section extension and localization extension


    1. Cannot use "." as the key name
    -


        { "data.2": "value" }


    2. Cannot use "$" as the prefix in key names
    -


        { "$data": "value" }

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:STORE [CREATE]

    Properties:
        url: /platform/admin/namespaces/{namespace}/sections

        method: POST

        tags: ["Section"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH] or [BEARER_AUTH]

        body: (body) OPTIONAL SectionCreate in body

        namespace: (namespace) REQUIRED str in path

        store_id: (storeId) REQUIRED str in query

    Responses:
        201: Created - FullSectionInfo (successful operation)

        400: Bad Request - ErrorEntity (20027: Invalid time range | 30021: Default language [{language}] required | 30023: Catalog plugin grpc server address required)

        404: Not Found - ErrorEntity (30141: Store [{storeId}] does not exist in namespace [{namespace}] | 30641: View [{viewId}] does not exist in namespace [{namespace}])

        409: Conflict - ErrorEntity (30173: Published store can't modify content)

        422: Unprocessable Entity - ValidationErrorEntity (20002: validation error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = CreateSection.create(
        store_id=store_id,
        body=body,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(CreateSection)
async def create_section_async(
    store_id: str,
    body: Optional[SectionCreate] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Create a section (createSection)

    This API is used to create a section.

    Other detail info:

      * Required permission : resource="ADMIN:NAMESPACE:{namespace}:STORE", action=1 (CREATE)
      *  Returns : created a section



    ## Restrictions for section extension and localization extension


    1. Cannot use "." as the key name
    -


        { "data.2": "value" }


    2. Cannot use "$" as the prefix in key names
    -


        { "$data": "value" }

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:STORE [CREATE]

    Properties:
        url: /platform/admin/namespaces/{namespace}/sections

        method: POST

        tags: ["Section"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH] or [BEARER_AUTH]

        body: (body) OPTIONAL SectionCreate in body

        namespace: (namespace) REQUIRED str in path

        store_id: (storeId) REQUIRED str in query

    Responses:
        201: Created - FullSectionInfo (successful operation)

        400: Bad Request - ErrorEntity (20027: Invalid time range | 30021: Default language [{language}] required | 30023: Catalog plugin grpc server address required)

        404: Not Found - ErrorEntity (30141: Store [{storeId}] does not exist in namespace [{namespace}] | 30641: View [{viewId}] does not exist in namespace [{namespace}])

        409: Conflict - ErrorEntity (30173: Published store can't modify content)

        422: Unprocessable Entity - ValidationErrorEntity (20002: validation error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = CreateSection.create(
        store_id=store_id,
        body=body,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(DeleteSection)
def delete_section(
    section_id: str,
    store_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Delete a section (deleteSection)

    This API is used to delete s section.

    Other detail info:

      * Required permission : resource="ADMIN:NAMESPACE:{namespace}:STORE", action=8 (DELETE)

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:STORE [DELETE]

    Properties:
        url: /platform/admin/namespaces/{namespace}/sections/{sectionId}

        method: DELETE

        tags: ["Section"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH] or [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        section_id: (sectionId) REQUIRED str in path

        store_id: (storeId) REQUIRED str in query

    Responses:
        204: No Content - (Delete section successfully)

        404: Not Found - ErrorEntity (30141: Store [{storeId}] does not exist in namespace [{namespace}] | 30741: Section [{sectionId}] does not exist in namespace [{namespace}])

        409: Conflict - ErrorEntity (30173: Published store can't modify content)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = DeleteSection.create(
        section_id=section_id,
        store_id=store_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(DeleteSection)
async def delete_section_async(
    section_id: str,
    store_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Delete a section (deleteSection)

    This API is used to delete s section.

    Other detail info:

      * Required permission : resource="ADMIN:NAMESPACE:{namespace}:STORE", action=8 (DELETE)

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:STORE [DELETE]

    Properties:
        url: /platform/admin/namespaces/{namespace}/sections/{sectionId}

        method: DELETE

        tags: ["Section"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH] or [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        section_id: (sectionId) REQUIRED str in path

        store_id: (storeId) REQUIRED str in query

    Responses:
        204: No Content - (Delete section successfully)

        404: Not Found - ErrorEntity (30141: Store [{storeId}] does not exist in namespace [{namespace}] | 30741: Section [{sectionId}] does not exist in namespace [{namespace}])

        409: Conflict - ErrorEntity (30173: Published store can't modify content)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = DeleteSection.create(
        section_id=section_id,
        store_id=store_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(GetSection)
def get_section(
    section_id: str,
    store_id: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get a section (getSection)

    This API is used to get a section.

    Other detail info:

      * Required permission : resource="ADMIN:NAMESPACE:{namespace}:STORE", action=2 (READ)
      *  Returns : section data

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:STORE [READ]

    Properties:
        url: /platform/admin/namespaces/{namespace}/sections/{sectionId}

        method: GET

        tags: ["Section"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH] or [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        section_id: (sectionId) REQUIRED str in path

        store_id: (storeId) OPTIONAL str in query

    Responses:
        200: OK - FullSectionInfo (successful operation)

        404: Not Found - ErrorEntity (30141: Store [{storeId}] does not exist in namespace [{namespace}] | 30142: Published store does not exist in namespace [{namespace}] | 30741: Section [{sectionId}] does not exist in namespace [{namespace}])
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetSection.create(
        section_id=section_id,
        store_id=store_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(GetSection)
async def get_section_async(
    section_id: str,
    store_id: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get a section (getSection)

    This API is used to get a section.

    Other detail info:

      * Required permission : resource="ADMIN:NAMESPACE:{namespace}:STORE", action=2 (READ)
      *  Returns : section data

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:STORE [READ]

    Properties:
        url: /platform/admin/namespaces/{namespace}/sections/{sectionId}

        method: GET

        tags: ["Section"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH] or [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        section_id: (sectionId) REQUIRED str in path

        store_id: (storeId) OPTIONAL str in query

    Responses:
        200: OK - FullSectionInfo (successful operation)

        404: Not Found - ErrorEntity (30141: Store [{storeId}] does not exist in namespace [{namespace}] | 30142: Published store does not exist in namespace [{namespace}] | 30741: Section [{sectionId}] does not exist in namespace [{namespace}])
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetSection.create(
        section_id=section_id,
        store_id=store_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(PublicListActiveSections)
def public_list_active_sections(
    user_id: str,
    language: Optional[str] = None,
    region: Optional[str] = None,
    store_id: Optional[str] = None,
    view_id: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """List active section contents (publicListActiveSections)

    This API is used to list active section contents.

    Other detail info:

      * Required permission : resource=NAMESPACE:{namespace}:USER:{userId}:STORE, action=2 (READ)
      *  Optional permission : resource="PREVIEW", action=1(CREATE) (user with this permission can view draft store sections)
      *  Optional permission : resource="SANDBOX", action=1(CREATE) (user with this permission can view draft store sections)
      *  Returns : active section contents

    Properties:
        url: /platform/public/namespaces/{namespace}/users/{userId}/sections

        method: GET

        tags: ["Section"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

        language: (language) OPTIONAL str in query

        region: (region) OPTIONAL str in query

        store_id: (storeId) OPTIONAL str in query

        view_id: (viewId) OPTIONAL str in query

    Responses:
        200: OK - List[SectionInfo] (successful operation)

        404: Not Found - ErrorEntity (30141: Store [{storeId}] does not exist in namespace [{namespace}] | 30142: Published store does not exist in namespace [{namespace}])
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = PublicListActiveSections.create(
        user_id=user_id,
        language=language,
        region=region,
        store_id=store_id,
        view_id=view_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(PublicListActiveSections)
async def public_list_active_sections_async(
    user_id: str,
    language: Optional[str] = None,
    region: Optional[str] = None,
    store_id: Optional[str] = None,
    view_id: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """List active section contents (publicListActiveSections)

    This API is used to list active section contents.

    Other detail info:

      * Required permission : resource=NAMESPACE:{namespace}:USER:{userId}:STORE, action=2 (READ)
      *  Optional permission : resource="PREVIEW", action=1(CREATE) (user with this permission can view draft store sections)
      *  Optional permission : resource="SANDBOX", action=1(CREATE) (user with this permission can view draft store sections)
      *  Returns : active section contents

    Properties:
        url: /platform/public/namespaces/{namespace}/users/{userId}/sections

        method: GET

        tags: ["Section"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

        language: (language) OPTIONAL str in query

        region: (region) OPTIONAL str in query

        store_id: (storeId) OPTIONAL str in query

        view_id: (viewId) OPTIONAL str in query

    Responses:
        200: OK - List[SectionInfo] (successful operation)

        404: Not Found - ErrorEntity (30141: Store [{storeId}] does not exist in namespace [{namespace}] | 30142: Published store does not exist in namespace [{namespace}])
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = PublicListActiveSections.create(
        user_id=user_id,
        language=language,
        region=region,
        store_id=store_id,
        view_id=view_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(PurgeExpiredSection)
def purge_expired_section(
    store_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """purge expired section (purgeExpiredSection)

    This API is used to purge expired section.

    Other detail info:

      * Required permission : resource="ADMIN:NAMESPACE:{namespace}:STORE", action=8 (DELETE)

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:STORE [DELETE]

    Properties:
        url: /platform/admin/namespaces/{namespace}/sections/purge/expired

        method: DELETE

        tags: ["Section"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH] or [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        store_id: (storeId) REQUIRED str in query

    Responses:
        204: No Content - (purge expired section successfully)

        404: Not Found - ErrorEntity (30141: Store [{storeId}] does not exist in namespace [{namespace}])
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = PurgeExpiredSection.create(
        store_id=store_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(PurgeExpiredSection)
async def purge_expired_section_async(
    store_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """purge expired section (purgeExpiredSection)

    This API is used to purge expired section.

    Other detail info:

      * Required permission : resource="ADMIN:NAMESPACE:{namespace}:STORE", action=8 (DELETE)

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:STORE [DELETE]

    Properties:
        url: /platform/admin/namespaces/{namespace}/sections/purge/expired

        method: DELETE

        tags: ["Section"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH] or [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        store_id: (storeId) REQUIRED str in query

    Responses:
        204: No Content - (purge expired section successfully)

        404: Not Found - ErrorEntity (30141: Store [{storeId}] does not exist in namespace [{namespace}])
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = PurgeExpiredSection.create(
        store_id=store_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(QuerySections)
def query_sections(
    end: Optional[str] = None,
    limit: Optional[int] = None,
    offset: Optional[int] = None,
    start: Optional[str] = None,
    store_id: Optional[str] = None,
    view_id: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """List sections (querySections)

    This API is used to query sections.

    Other detail info:

      * Required permission : resource="ADMIN:NAMESPACE:{namespace}:STORE", action=2 (READ)
      *  Returns : paginated sections

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:STORE [READ]

    Properties:
        url: /platform/admin/namespaces/{namespace}/sections

        method: GET

        tags: ["Section"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH] or [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        end: (end) OPTIONAL str in query

        limit: (limit) OPTIONAL int in query

        offset: (offset) OPTIONAL int in query

        start: (start) OPTIONAL str in query

        store_id: (storeId) OPTIONAL str in query

        view_id: (viewId) OPTIONAL str in query

    Responses:
        200: OK - SectionPagingSlicedResult (successful operation)

        404: Not Found - ErrorEntity (30141: Store [{storeId}] does not exist in namespace [{namespace}] | 30142: Published store does not exist in namespace [{namespace}])

        422: Unprocessable Entity - ValidationErrorEntity (20002: validation error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = QuerySections.create(
        end=end,
        limit=limit,
        offset=offset,
        start=start,
        store_id=store_id,
        view_id=view_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(QuerySections)
async def query_sections_async(
    end: Optional[str] = None,
    limit: Optional[int] = None,
    offset: Optional[int] = None,
    start: Optional[str] = None,
    store_id: Optional[str] = None,
    view_id: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """List sections (querySections)

    This API is used to query sections.

    Other detail info:

      * Required permission : resource="ADMIN:NAMESPACE:{namespace}:STORE", action=2 (READ)
      *  Returns : paginated sections

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:STORE [READ]

    Properties:
        url: /platform/admin/namespaces/{namespace}/sections

        method: GET

        tags: ["Section"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH] or [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        end: (end) OPTIONAL str in query

        limit: (limit) OPTIONAL int in query

        offset: (offset) OPTIONAL int in query

        start: (start) OPTIONAL str in query

        store_id: (storeId) OPTIONAL str in query

        view_id: (viewId) OPTIONAL str in query

    Responses:
        200: OK - SectionPagingSlicedResult (successful operation)

        404: Not Found - ErrorEntity (30141: Store [{storeId}] does not exist in namespace [{namespace}] | 30142: Published store does not exist in namespace [{namespace}])

        422: Unprocessable Entity - ValidationErrorEntity (20002: validation error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = QuerySections.create(
        end=end,
        limit=limit,
        offset=offset,
        start=start,
        store_id=store_id,
        view_id=view_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(UpdateSection)
def update_section(
    section_id: str,
    store_id: str,
    body: Optional[SectionUpdate] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Update s section (updateSection)

    This API is used to update s section.

    Other detail info:

      * Required permission : resource="ADMIN:NAMESPACE:{namespace}:STORE", action=4 (UPDATE)
      *  Returns : updated section data



    ## Restrictions for section extension and localization extension


    1. Cannot use "." as the key name
    -


        { "data.2": "value" }


    2. Cannot use "$" as the prefix in key names
    -


        { "$data": "value" }

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:STORE [UPDATE]

    Properties:
        url: /platform/admin/namespaces/{namespace}/sections/{sectionId}

        method: PUT

        tags: ["Section"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH] or [BEARER_AUTH]

        body: (body) OPTIONAL SectionUpdate in body

        namespace: (namespace) REQUIRED str in path

        section_id: (sectionId) REQUIRED str in path

        store_id: (storeId) REQUIRED str in query

    Responses:
        200: OK - FullSectionInfo (successful operation)

        400: Bad Request - ErrorEntity (20027: Invalid time range | 30021: Default language [{language}] required | 30023: Catalog plugin grpc server address required)

        404: Not Found - ErrorEntity (30141: Store [{storeId}] does not exist in namespace [{namespace}] | 30641: View [{viewId}] does not exist in namespace [{namespace}] | 30741: Section [{sectionId}] does not exist in namespace [{namespace}])

        409: Conflict - ErrorEntity (30173: Published store can't modify content)

        422: Unprocessable Entity - ValidationErrorEntity (20002: validation error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = UpdateSection.create(
        section_id=section_id,
        store_id=store_id,
        body=body,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(UpdateSection)
async def update_section_async(
    section_id: str,
    store_id: str,
    body: Optional[SectionUpdate] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Update s section (updateSection)

    This API is used to update s section.

    Other detail info:

      * Required permission : resource="ADMIN:NAMESPACE:{namespace}:STORE", action=4 (UPDATE)
      *  Returns : updated section data



    ## Restrictions for section extension and localization extension


    1. Cannot use "." as the key name
    -


        { "data.2": "value" }


    2. Cannot use "$" as the prefix in key names
    -


        { "$data": "value" }

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:STORE [UPDATE]

    Properties:
        url: /platform/admin/namespaces/{namespace}/sections/{sectionId}

        method: PUT

        tags: ["Section"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH] or [BEARER_AUTH]

        body: (body) OPTIONAL SectionUpdate in body

        namespace: (namespace) REQUIRED str in path

        section_id: (sectionId) REQUIRED str in path

        store_id: (storeId) REQUIRED str in query

    Responses:
        200: OK - FullSectionInfo (successful operation)

        400: Bad Request - ErrorEntity (20027: Invalid time range | 30021: Default language [{language}] required | 30023: Catalog plugin grpc server address required)

        404: Not Found - ErrorEntity (30141: Store [{storeId}] does not exist in namespace [{namespace}] | 30641: View [{viewId}] does not exist in namespace [{namespace}] | 30741: Section [{sectionId}] does not exist in namespace [{namespace}])

        409: Conflict - ErrorEntity (30173: Published store can't modify content)

        422: Unprocessable Entity - ValidationErrorEntity (20002: validation error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = UpdateSection.create(
        section_id=section_id,
        store_id=store_id,
        body=body,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )
